# Copyright 2014 healthcheck-as-a-service authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from flask import Flask, request
from flask.ext.admin import Admin
from terminaltables import AsciiTable

from raven.contrib.flask import Sentry

from healthcheck import admin as hadmin
from healthcheck import auth
from healthcheck.storage import ItemNotFoundError

import json
import inspect
import os


app = Flask(__name__)
app.debug = os.environ.get("API_DEBUG", "0") in ("True", "true", "1")

SENTRY_DSN = os.environ.get("SENTRY_DSN")
if SENTRY_DSN:
    app.config['SENTRY_DSN'] = SENTRY_DSN
    sentry = Sentry(app)

admin = Admin(app, name="Tsuru healthcheck service")
admin.add_view(
    hadmin.HealthcheckAdmin(name='healthchecks', endpoint='healthchecks'))
admin.add_view(hadmin.UrlAdmin(name='urls', endpoint='urls'))
admin.add_view(hadmin.WatcherAdmin(name='watchers', endpoint='watchers'))


def get_manager():
    from healthcheck.backends import Zabbix
    managers = {
        "zabbix": Zabbix(),
    }
    manager = os.environ.get("API_MANAGER", "zabbix")
    manager_class = managers.get(manager)
    if manager_class:
        return manager_class
    raise ValueError("{0} is not a valid manager".format(manager))


@app.route("/resources/<name>/url", methods=["POST"])
@auth.required
def add_url(name):
    if not request.data:
        return "url is required", 400
    data = json.loads(request.data)
    if "url" not in data:
        return "url is required", 400
    data["name"] = name
    get_manager().add_url(**data)
    return "", 201


@app.route("/resources/<name>/url", methods=["DELETE"])
@auth.required
def remove_url(name):
    if not request.data:
        return "url is required", 400
    data = json.loads(request.data)
    if "url" not in data:
        return "url is required", 400
    data["name"] = name
    try:
        get_manager().remove_url(**data)
    except ItemNotFoundError:
        return "URL not found.", 404
    return "", 204


@app.route("/resources/<name>/url", methods=["GET"])
@auth.required
def list_urls(name):
    urls = get_manager().list_urls(name)
    table_urls = [["Url", "Comment"]]
    table_urls.extend(urls)
    table = AsciiTable(table_urls)
    return table.table, 200


@app.route("/resources/<name>/watcher", methods=["POST"])
@auth.required
def add_watcher(name):
    if not request.data:
        return "watcher is required", 400

    data = json.loads(request.data)

    if "watcher" not in data:
        return "watcher is required", 400

    watcher = data["watcher"]
    get_manager().add_watcher(name, watcher)

    return "", 201


# This route only exists for compatibility with old plugin versions and its an
# alias to /resources/<name>/watcher/<watcher>
@auth.required
@app.route("/resources/<name>/<ignored>/watcher/<watcher>", methods=["DELETE"])
def remove_watcher_compat(name, ignored, watcher):
    get_manager().remove_watcher(name, watcher)
    return "", 204


@app.route("/resources/<name>/watcher/<watcher>", methods=["DELETE"])
@auth.required
def remove_watcher(name, watcher):
    get_manager().remove_watcher(name, watcher)
    return "", 204


@app.route("/resources/<name>/watcher", methods=["GET"])
@auth.required
def list_watchers(name):
    watchers = get_manager().list_watchers(name)
    return json.dumps(watchers), 200


@app.route("/resources", methods=["POST"])
@auth.required
def new():
    name = request.form.get("name")
    get_manager().new(name)
    return "", 201


@app.route("/resources/<name>", methods=["DELETE"])
@auth.required
def remove(name):
    get_manager().remove(name)
    return "", 204


@app.route("/plugin", methods=["GET"])
def plugin():
    from healthcheck import plugin
    source = inspect.getsource(plugin)
    return source, 200


@app.route("/resources/<name>/bind", methods=["POST"])
def bind_unit(name):
    return "", 201


@app.route("/resources/<name>/bind-app", methods=["POST"])
def bind_app(name):
    return "{}", 200


@app.route("/resources/<name>/bind", methods=["DELETE"])
def unbind_unit(name):
    return "", 200


@app.route("/resources/<name>/bind-app", methods=["DELETE"])
def unbind_app(name):
    return "", 200


if __name__ == "__main__":
    app.run(host="0.0.0.0", port=8888, debug=True)



#!/usr/bin/env python

# Copyright 2014 healthcheck-as-a-service authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import json
import os
import sys
import urllib2


def get_env(name):
    env = os.environ.get(name)
    if not env:
        sys.stderr.write("ERROR: missing {}\n".format(name))
        sys.exit(5)
    return env


class Request(urllib2.Request):

    def __init__(self, method, *args, **kwargs):
        self._method = method
        urllib2.Request.__init__(self, *args, **kwargs)

    def get_method(self):
        return self._method


def proxy_request(service_name, instance_name, method, path, body=None, headers=None):
    target = get_env("TSURU_TARGET").rstrip("/")
    token = get_env("TSURU_TOKEN")

    if not target.startswith("http://") and not target.startswith("https://"):
        target = "http://{}".format(target)

    url = "{}/services/{}/proxy/{}?callback=/resources/{}/{}".format(target, service_name, instance_name, instance_name, path.lstrip("/"))

    if body:
        body = json.dumps(body)

    request = Request(method, url, data=body)
    request.add_header("Authorization", "bearer " + token)

    if headers:
        for header, value in headers.items():
            request.add_header(header, value)

    return urllib2.urlopen(request, timeout=30)


def add_url(service_name, name, url, expected_string=None, comment=None):
    """
    add-url add a new url checker to the given instance. Usage:

        add-url <service-name> <instance-name> <url> [expected_string] [comment]

    expected_string is an optional parameter that represents the string that
    the healthcheck should expect to find in the body of the response. Example:

        tsuru {plugin_name} add-url hcaas mysite http://mysite.com/hc WORKING

    comment is an optional parameter that refers to what to do when a trigger
    is triggered. Example:

        tsuru {plugin_name} add-url hcaas mysite http://mysite.com/hc 'restart the app'

    """
    data = {
        "url": url,
    }
    if expected_string:
        data["expected_string"] = expected_string
    if comment:
        data["comment"] = comment
    headers = {
        "Content-Type": "application/json",
        "Accept": "text/plain"
    }
    proxy_request(service_name, name, "POST", "/url", data, headers)
    msg = "url {} successfully added!\n".format(url)
    sys.stdout.write(msg)


def remove_url(service_name, name, url):
    """
    remove-url removes the specified url checker from the specified instance.
    Usage:

        remove-url <service_name> <name> <url>

    Example:

        tsuru {plugin_name} remove-url hcaas mysite http://mysite.com/hc
    """
    body = {"url": url}
    headers = {"Content-Type": "application/json"}
    try:
        proxy_request(service_name, name, "DELETE", "/url", body=body, headers=headers)
    except urllib2.HTTPError:
        sys.stdout.write("URL %s not found.\n" % url)
        return
    msg = "url {} successfully removed!\n".format(url)
    sys.stdout.write(msg)


def list_urls(service_name, name):
    """
    list-urls list all urls from an instance.
    Usage:

        list-urls <service_name> <name>

    Example:

        tsuru {plugin_name} list-urls hcaas mysite
    """
    url = "/url"
    headers = {"Content-Type": "application/json"}
    response = proxy_request(service_name, name, "GET", url, "", headers)
    urls = response.read()
    sys.stdout.write(urls + "\n")


def add_watcher(service_name, name, watcher):
    """
    add-watcher creates a new watcher for the given monitoring instance. A
    watcher is an email address that will receive notifications for this
    instance. Usage:

        add-watcher <service_name> <instance-name> <email>

    Example:

        tsuru {plugin_name} add-watcher hcaas mysite mysite+monit@mycompany.com
    """
    data = {
        "watcher": watcher,
    }
    headers = {
        "Content-Type": "application/json",
        "Accept": "text/plain"
    }
    proxy_request(service_name, name, "POST", "/watcher", data, headers)
    msg = "watcher {} successfully added!\n".format(watcher)
    sys.stdout.write(msg)


def remove_watcher(service_name, name, watcher):
    """
    remove-watcher removes the specified watcher from the given monitoring
    instance. Usage:

        remove-watcher <service_name> <instance-name> <email>

    Example:

        tsuru {plugin_name} remove-watcher hcaas mysite mysite+monit@mycompany.com
    """
    url = "/watcher/{}".format(watcher)
    proxy_request(service_name, name, "DELETE", url)
    msg = "watcher {} successfully removed!\n".format(watcher)
    sys.stdout.write(msg)


def list_watchers(service_name, name):
    """
    list-watchers list all watchers from an instance.
    Usage:

        list-watchers <service_name> <name>

    Example:

        tsuru {plugin_name} list-watchers hcaas mysite
    """
    url = '/watcher'
    headers = {"Content-Type": "application/json"}
    response = proxy_request(service_name, name, "GET", url, "", headers)
    watchers_json = response.read()
    watchers = json.loads(watchers_json)
    for watcher in watchers:
        sys.stdout.write(watcher + "\n")


def show_help(command_name=None, exit=0):
    """
    help displays the help of the specified command. Usage:

        help <command-name>

    Example:

        tsuru {plugin_name} help [command-name]

    The command-name is optional, when omitted the plugin will list all
    available commands.
    """
    plugin_name = os.environ.get("TSURU_PLUGIN_NAME", "hc")
    commands = _get_commands()
    if command_name and command_name in commands:
        command = commands[command_name]
        doc = command.__doc__.format(plugin_name=plugin_name)
        sys.stderr.write(doc.rstrip() + "\n")
        sys.exit(exit)
    msg = "Usage: tsuru {plugin_name} command [args]\n\n"
    sys.stderr.write(msg.format(plugin_name=plugin_name))
    sys.stderr.write("Available commands:\n")

    for name in sorted(commands.keys()):
        if name != "help":
            sys.stderr.write("  {}\n".format(name))
    sys.stderr.write("  help\n")

    msg = "Use tsuru {plugin_name} help <commandname> to get more details."
    sys.stderr.write("\n" + msg.format(plugin_name=plugin_name) + "\n")
    sys.exit(exit)


def _get_commands():
    return {
        "add-url": add_url,
        "remove-url": remove_url,
        "list-urls": list_urls,
        "add-watcher": add_watcher,
        "remove-watcher": remove_watcher,
        "list-watchers": list_watchers,
        "help": show_help,
    }


def command(command_name):
    commands = _get_commands()
    if command_name in commands:
        return commands[command_name]
    show_help(exit=2)


def main(cmd, *args):
    try:
        command(cmd)(*args)
    except TypeError:
        show_help(cmd, exit=2)


if __name__ == "__main__":
    if len(sys.argv) < 2:
        show_help(exit=2)
    main(sys.argv[1], *sys.argv[2:])



# -*- coding: utf-8 -*-
# Copyright 2015 healthcheck-as-a-service authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import os
import unittest

import mock

from healthcheck.backends import (WatcherAlreadyRegisteredError,
                                  WatcherNotInInstanceError, get_value)
from healthcheck.storage import Item, User, HealthCheck, UserNotFoundError


class ZabbixTest(unittest.TestCase):

    @mock.patch("healthcheck.storage.MongoStorage")
    @mock.patch("pyzabbix.ZabbixAPI")
    def setUp(self, zabbix_mock, mongo_mock):
        os.environ["ZABBIX_URL"] = self.url = "http://zbx.com"
        os.environ["ZABBIX_USER"] = self.user = "user"
        os.environ["ZABBIX_PASSWORD"] = self.password = "pass"
        os.environ["ZABBIX_HOST"] = "1"
        os.environ["ZABBIX_HOST_GROUP"] = "2"
        zapi_mock = mock.Mock()
        zapi_mock.trigger.get.return_value = {"result": [{"comments": "teste"}]}
        zabbix_mock.return_value = zapi_mock

        instance_mock = mock.Mock()
        mongo_mock.return_value = instance_mock

        from healthcheck.backends import Zabbix
        self.backend = Zabbix()
        zabbix_mock.assert_called_with(self.url)
        zapi_mock.login.assert_called_with(self.user, self.password)

        mongo_mock.assert_called_with()
        instance_mock.conn.assert_called_with()
        self.backend.storage = mock.Mock()

    def test_get_value(self):
        url = get_value("ZABBIX_URL")
        self.assertEqual(self.url, url)

    def test_get_value_failure(self):
        with self.assertRaises(Exception) as cm:
            get_value("ZABBIX_URL_URL_URL_URL_")
        exc = cm.exception
        expected = (u"You must define the ZABBIX_URL_URL_URL_URL_ environment"
                    u" variable.")
        self.assertEqual((expected,), exc.args)

    def test_add_url(self):
        url = "http://mysite.com/012345678901234567890123456789012345678"
        hc_name = "hc_name"
        item_name = "hc for {}".format(url)

        self.backend.zapi.httptest.create.return_value = {"httptestids": [1]}
        self.backend.zapi.trigger.create.return_value = {"triggerids": [1]}
        old_add_action = self.backend._add_action
        self.backend._add_action = mock.Mock()
        hmock = mock.Mock(host_id="1", group_id=13)
        self.backend.storage.find_healthcheck_by_name.return_value = hmock

        self.backend.add_url(hc_name, url)

        self.backend.storage.find_healthcheck_by_name.assert_called_with(
            hc_name)
        self.backend.zapi.httptest.create.assert_called_with(
            name=item_name,
            steps=[{
                "name": item_name,
                "url": url,
                "status_codes": 200,
                "no": 1,
            }],
            hostid="1",
            retries=3,
        )
        expression = ("{{hc_name:web.test.rspcode[{item_name},"
                      "{item_name}].last()}}#200 | {{hc_name:web.test.fail["
                      "{item_name}].last()}}#0 & {{hc_name:web.test.error["
                      "{item_name}].str(required pattern not found)}}=1")
        self.backend.zapi.trigger.create.assert_called_with(
            description="trigger for url {}".format(url),
            expression=expression.format(item_name=item_name),
            priority=5,
            comments=None,
        )
        self.assertTrue(self.backend.storage.add_item.called)
        self.backend._add_action.assert_called_with(url, 1, 13)
        self.backend._add_action = old_add_action

    def test_add_url_expected_string(self):
        url = "http://mysite.com"
        hc_name = "hc_name"
        item_name = "hc for {}".format(url)

        self.backend.zapi.httptest.create.return_value = {"httptestids": [1]}
        self.backend.zapi.trigger.create.return_value = {"triggerids": [1]}
        old_add_action = self.backend._add_action

        def set_old_add_action():
            self.backend._add_action = old_add_action
        self.addCleanup(set_old_add_action)
        self.backend._add_action = mock.Mock()
        hmock = mock.Mock(host_id="1", group_id=13)
        self.backend.storage.find_healthcheck_by_name.return_value = hmock

        self.backend.add_url(hc_name, url, expected_string="WORKING")

        self.backend.storage.find_healthcheck_by_name.assert_called_with(
            hc_name)
        self.backend.zapi.httptest.create.assert_called_with(
            name=item_name,
            steps=[{
                "name": item_name,
                "url": url,
                "status_codes": 200,
                "no": 1,
                "required": "WORKING",
            }],
            hostid="1",
            retries=3,
        )

    def test_add_url_comment(self):
        url = "http://mysite.com"
        hc_name = "hc_name"

        self.backend.zapi.httptest.create.return_value = {"httptestids": [1]}
        self.backend.zapi.trigger.create.return_value = {"triggerids": [1]}
        old_add_action = self.backend._add_action

        def set_old_add_action():
            self.backend._add_action = old_add_action
        self.addCleanup(set_old_add_action)
        self.backend._add_action = mock.Mock()
        hmock = mock.Mock(host_id="1", group_id=13)
        self.backend.storage.find_healthcheck_by_name.return_value = hmock

        self.backend.add_url(hc_name, url, comment="http://test.com")
        self.backend.zapi.trigger.create.assert_called_with(
            description="trigger for url {}".format(url),
            expression='{hc_name:web.test.rspcode[hc for http://mysite.com,hc for http://mysite.com].last()}#200 \
| {hc_name:web.test.fail[hc for http://mysite.com].last()}#0 & {hc_name:web.test.error\
[hc for http://mysite.com].str(required pattern not found)}=1',
            priority=5,
            comments="http://test.com",
        )

    def test_add_url_big_url(self):
        url = "http://mysite.com/01234567890123456789012345" \
              "67890123456789012345678901234567890123456789"
        hc_name = "hc_name"
        item_name = "hc for http://mysite.com/" \
                    "012345678901234567890123456789012345..."

        self.backend.zapi.httptest.create.return_value = {"httptestids": [1]}
        self.backend.zapi.trigger.create.return_value = {"triggerids": [1]}
        old_add_action = self.backend._add_action
        self.backend._add_action = mock.Mock()
        hmock = mock.Mock(host_id="1", group_id=13)
        self.backend.storage.find_healthcheck_by_name.return_value = hmock

        self.backend.add_url(hc_name, url)

        self.backend.storage.find_healthcheck_by_name.assert_called_with(
            hc_name)
        self.backend.zapi.httptest.create.assert_called_with(
            name=item_name,
            steps=[{
                "name": item_name,
                "url": url,
                "status_codes": 200,
                "no": 1,
            }],
            hostid="1",
            retries=3,
        )
        expression = ("{{hc_name:web.test.rspcode[{item_name},"
                      "{item_name}].last()}}#200 | {{hc_name:web.test.fail["
                      "{item_name}].last()}}#0 & {{hc_name:web.test.error["
                      "{item_name}].str(required pattern not found)}}=1")
        self.backend.zapi.trigger.create.assert_called_with(
            description="trigger for url {}".format(url),
            expression=expression.format(item_name=item_name),
            priority=5,
            comments=None,
        )
        self.assertTrue(self.backend.storage.add_item.called)
        self.backend._add_action.assert_called_with(url, 1, 13)
        self.backend._add_action = old_add_action

    def test_remove_url(self):
        url = "http://mysite.com"
        item_id = 1
        trigger_id = 1
        action_id = 8
        item = Item(
            url,
            item_id=item_id,
            trigger_id=trigger_id,
            action_id=action_id
        )
        self.backend.storage.find_item_by_url.return_value = item
        old_action = self.backend._remove_action
        self.backend._remove_action = mock.Mock()

        self.backend.remove_url("hc_name", url)

        self.backend._remove_action.assert_called_with(8)
        self.backend.zapi.httptest.delete.assert_called_with(item_id)
        self.backend._remove_action = old_action
        self.backend.storage.remove_item.assert_called_with(item)

    def test_add_watcher(self):
        email = "andrews@corp.globo.com"
        name = "hc_name"
        hmock = mock.Mock(group_id="someid")
        self.backend.storage.find_user_by_email.side_effect = UserNotFoundError
        self.backend.storage.find_healthcheck_by_name.return_value = hmock
        self.backend.zapi.user.create.return_value = {"userids": ["123"]}

        self.backend.add_watcher(name, email)

        self.backend.storage.find_healthcheck_by_name.assert_called_with(name)
        self.backend.zapi.user.create.assert_called_with(
            alias=email,
            passwd="",
            usrgrps=["someid"],
            user_medias=[{
                "mediatypeid": "1",
                "sendto": email,
                "active": 0,
                "severity": 63,
                "period": "1-7,00:00-24:00",
            }],
        )
        self.assertTrue(self.backend.storage.add_user.called)

    def test_add_watcher_user_to_group(self):
        email = "andrews@corp.globo.com"
        name = "hc_name"
        hmock = mock.Mock(group_id="someid")
        umock = mock.Mock(id="userid3")
        usersmock = [mock.Mock(id="userid1"), mock.Mock(id="userid2")]
        self.backend.storage.find_user_by_email.return_value = umock
        self.backend.storage.find_users_by_group.return_value = usersmock
        self.backend.storage.find_healthcheck_by_name.return_value = hmock

        self.backend.add_watcher(name, email)

        self.backend.storage.find_healthcheck_by_name.assert_called_with(name)
        self.backend.storage.find_user_by_email.assert_called_with(email)
        self.backend.storage.find_users_by_group.assert_called_with("someid")
        self.backend.zapi.usergroup.update.assert_called_with(
            usrgrpid="someid",
            userids=["userid1", "userid2", "userid3"],
        )
        self.backend.storage.add_user_to_group.assert_called_with(umock,
                                                                  "someid")

    def test_add_watcher_user_already_in_the_group(self):
        email = "andrews@corp.globo.com"
        name = "hc_name"
        hmock = mock.Mock(group_id="someid")
        umock = mock.Mock(id="userid2")
        usersmock = [mock.Mock(id="userid1"), mock.Mock(id="userid2")]
        self.backend.storage.find_user_by_email.return_value = umock
        self.backend.storage.find_users_by_group.return_value = usersmock
        self.backend.storage.find_healthcheck_by_name.return_value = hmock
        with self.assertRaises(WatcherAlreadyRegisteredError):
            self.backend.add_watcher(name, email)

    def test_add_action(self):
        self.backend.zapi.action.create.return_value = {"actionids": ["1"]}

        self.backend._add_action("url", "8", "14")

        self.backend.zapi.action.create.assert_called_with(
            operations=[
                {
                    'mediatypeid': 0,
                    'esc_step_to': 1,
                    'esc_step_from': 1,
                    'operationtype': 0,
                    'esc_period': 0,
                    'evaltype': 0,
                    'opmessage': {'mediatypeid': '0', 'default_msg': 1},
                    'opmessage_grp': [
                        {'usrgrpid': '14'}
                    ]
                }
            ],
            status=0,
            name='action for url url',
            esc_period=3600,
            def_shortdata=("hcaas {HOST.NAME} #{EVENT.ID} {TRIGGER.STATUS}: "
                           "{ITEM.VALUE3}"),
            def_longdata=("{TRIGGER.NAME}: {TRIGGER.STATUS}\r\n"
                          "HTTP status code: {ITEM.VALUE1}"),
            r_shortdata="hcaas {HOST.NAME} #{EVENT.ID} {TRIGGER.STATUS}",
            r_longdata=("{TRIGGER.NAME}: {TRIGGER.STATUS}\r\n"
                        "HTTP status code: {ITEM.VALUE1}"),
            evaltype=0,
            eventsource=0,
            conditions=[
                {
                    'operator': 7,
                    'conditiontype': 16,
                    'value': ''
                },
                {
                    'conditiontype': 5,
                    'value': '1'
                },
                {
                    'conditiontype': 2,
                    'value': '8'
                }
            ],
            recovery_msg=1
        )

    def test_add_group(self):
        name = "group name"
        host_group = "host group name"
        self.backend.zapi.usergroup.create.return_value = {"usrgrpids": [2]}
        self.backend._add_group(name, host_group)
        self.backend.zapi.usergroup.create.assert_called_with(
            name=name,
            rights={"permission": 2, "id": host_group},
        )

    def test_add_host(self):
        name = "host name"
        self.backend.zapi.host.create.return_value = {"hostids": [2]}

        result = self.backend._add_host(name, host_group="123")

        self.assertEqual(result, 2)

        expected = [{
            'ip': '127.0.0.1',
            'useip': 1,
            'dns': '',
            'main': 1,
            'type': 1,
            'port': '10050'
        }]
        self.backend.zapi.host.create.assert_called_with(
            interfaces=expected, host=name, groups=[{"groupid": "123"}])

    def test_remove_host(self):
        id = "id"
        self.backend._remove_host(id)
        self.backend.zapi.host.delete.assert_called_with("id")

    def test_new(self):
        name = "blah"

        old_add_group = self.backend._add_group
        self.backend._add_group = mock.Mock()

        old_add_host = self.backend._add_host
        self.backend._add_host = mock.Mock()

        self.backend.new(name)

        self.backend._add_group.assert_called_with(name, "2")
        self.backend._add_group = old_add_group

        self.backend._add_host.assert_called_with(name, "2")
        self.backend._add_host = old_add_host

        self.assertTrue(self.backend.storage.add_healthcheck.called)

    def test_remove_group(self):
        self.backend._remove_group("id")
        self.backend.zapi.usergroup.delete.assert_called_with("id")

    def test_remove_action(self):
        self.backend._remove_action("id")
        self.backend.zapi.action.delete.assert_called_with("id")

    def test_remove_watcher(self):
        hmock = mock.Mock(group_id="group")
        user = User("123", "email@email.com", "group")
        self.backend.storage.find_healthcheck_by_name.return_value = hmock
        self.backend.storage.find_user_by_email.return_value = user
        self.backend.remove_watcher("healthcheck", user.email)
        self.backend.zapi.user.delete.assert_called_with("123")
        self.backend.storage.remove_user.assert_called_with(user)

    def test_remove_watcher_not_last_group(self):
        group = "group1"
        hmock = mock.Mock(group_id=group)
        user = User("123", "email@email.com", "group1", "group2")
        users = [mock.Mock(id="123"), mock.Mock(id="456"),
                 mock.Mock(id="789")]
        self.backend.storage.find_users_by_group.return_value = users
        self.backend.storage.find_user_by_email.return_value = user
        self.backend.storage.find_healthcheck_by_name.return_value = hmock
        self.backend.remove_watcher("healthcheck", user.email)
        self.backend.zapi.usergroup.update.assert_called_with(
            usrgrpid="group1",
            userids=["456", "789"],
        )
        self.backend.storage.remove_user_from_group.assert_called_with(user, group)

    def test_remove_watcher_not_in_healthcheck(self):
        hmock = mock.Mock(group_id="group1")
        user = User("123", "email@email.com", "group2")
        self.backend.storage.find_healthcheck_by_name.return_value = hmock
        self.backend.storage.find_user_by_email.return_value = user
        with self.assertRaises(WatcherNotInInstanceError):
            self.backend.remove_watcher("healthcheck", user.email)

    def test_remove(self):
        name = "blah"
        id = "someid"
        url = 'http://test.com/healthcheck'

        old_remove_url = self.backend.remove_url
        self.backend.remove_url = mock.Mock()
        old_remove_watcher = self.backend.remove_watcher
        self.backend.remove_watcher = mock.Mock()
        self.backend.storage.find_urls_by_healthcheck_name.return_value = [url]

        hmock = mock.Mock(group_id=id, host_id=id)
        self.backend.storage.find_healthcheck_by_name.return_value = hmock
        watchers = ['test@example.com']
        self.backend.storage.find_watchers_by_healthcheck_name.return_value = watchers
        user = User("123", "email@email.com", id)
        self.backend.storage.find_user_by_email.return_value = user

        self.backend.remove(name)

        self.backend.storage.find_urls_by_healthcheck_name.assert_called_with(name)
        self.backend.zapi.usergroup.delete.assert_called_with(id)
        self.backend.zapi.host.delete.assert_called_with(id)
        self.backend.storage.remove_healthcheck.assert_called_with(hmock)
        assert self.backend.remove_url.called
        assert self.backend.remove_watcher.called

        self.backend.remove_url = old_remove_url
        self.backend.remove_watcher = old_remove_watcher

    def test_remove_with_urls(self):
        name = "blah"
        url = "http://mysite.com"
        item_id = 1
        trigger_id = 2
        action_id = 3
        group_id = "4"
        host_id = "5"
        user_id = "6"
        item = Item(
            url,
            item_id=item_id,
            trigger_id=trigger_id,
            action_id=action_id
        )

        self.backend.storage.find_item_by_url.return_value = item
        self.backend.storage.find_urls_by_healthcheck_name.return_value = [url]
        hc = HealthCheck(name, group_id=group_id, host_id=host_id)
        self.backend.storage.find_healthcheck_by_name.return_value = hc
        watchers = ['test@example.com']
        self.backend.storage.find_watchers_by_healthcheck_name.return_value = watchers
        user = User(user_id, "email@email.com", group_id)
        self.backend.storage.find_user_by_email.return_value = user
        self.backend.zapi.trigger.get.return_value = [{"comments": "xxx"}]

        self.backend.remove(name)

        self.backend.zapi.usergroup.delete.assert_called_with(group_id)
        self.backend.zapi.host.delete.assert_called_with(host_id)
        self.backend.zapi.action.delete.assert_called_with(action_id)
        self.backend.zapi.httptest.delete.assert_called_with(item_id)
        self.backend.zapi.user.delete.assert_called_with(user_id)
        self.backend.storage.remove_healthcheck.assert_called_with(hc)
        self.backend.storage.remove_item.assert_called_with(item)
        self.backend.storage.find_healthcheck_by_name.assert_called_with(name)
        self.backend.storage.find_urls_by_healthcheck_name.assert_called_with(name)
        self.backend.storage.find_item_by_url.assert_called_with(url)

    def test_remove_with_urls_more_than_one_group(self):
        name = "blah"
        url = "http://mysite.com"
        item_id = 1
        trigger_id = 2
        action_id = 3
        group_id = "4"
        another_group_id = "4-2"
        host_id = "5"
        user_id = "6"
        item = Item(
            url,
            item_id=item_id,
            trigger_id=trigger_id,
            action_id=action_id
        )

        self.backend.storage.find_item_by_url.return_value = item
        self.backend.storage.find_urls_by_healthcheck_name.return_value = [url]
        hc = HealthCheck(name, group_id=group_id, host_id=host_id)
        self.backend.storage.find_healthcheck_by_name.return_value = hc
        watchers = ['test@example.com']
        self.backend.storage.find_watchers_by_healthcheck_name.return_value = watchers
        user = User(user_id, "email@email.com", group_id, another_group_id)
        self.backend.storage.find_user_by_email.return_value = user
        self.backend.storage.find_users_by_group.return_value = [user]
        self.backend.zapi.trigger.get.return_value = [{"comments": "xxx"}]

        self.backend.remove(name)

        self.backend.zapi.usergroup.delete.assert_called_with(group_id)
        self.backend.zapi.host.delete.assert_called_with(host_id)
        self.backend.zapi.action.delete.assert_called_with(action_id)
        self.backend.zapi.httptest.delete.assert_called_with(item_id)
        self.backend.zapi.usergroup.update.assert_called_with(usrgrpid=group_id, userids=[])
        self.backend.storage.remove_healthcheck.assert_called_with(hc)
        self.backend.storage.remove_item.assert_called_with(item)
        self.backend.storage.find_healthcheck_by_name.assert_called_with(name)
        self.backend.storage.find_urls_by_healthcheck_name.assert_called_with(name)
        self.backend.storage.find_item_by_url.assert_called_with(url)
        self.backend.storage.find_users_by_group.assert_called_with(group_id)



# Copyright 2014 healthcheck-as-a-service authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import os


class Jsonable(object):

    def to_json(self):
        return self.__dict__


class HealthCheck(Jsonable):

    def __init__(self, name, **kwargs):
        self.name = name
        for key, value in kwargs.items():
            setattr(self, key, value)


class User(Jsonable):

    def __init__(self, id, email, *groups_id):
        self.email = email
        self.groups_id = groups_id
        self.id = id

    def __eq__(self, other):
        return other.email == self.email and \
            other.groups_id == self.groups_id and \
            other.id == self.id


class Item(Jsonable):

    def __init__(self, url, **kwargs):
        self.url = url
        for key, value in kwargs.items():
            setattr(self, key, value)

    def to_json(self):
        return self.__dict__


class MongoStorage(object):

    def __init__(self):
        self.database_name = os.environ.get("MONGODB_DATABASE", "hcapi")
        self.db = self.conn()[self.database_name]

    def conn(self):
        mongodb_uri = os.environ.get(
            "MONGODB_URI", "mongodb://localhost:27017/"
        )
        from pymongo import MongoClient
        return MongoClient(mongodb_uri)

    def add_item(self, item):
        self.db.items.insert(item.to_json())

    def find_item_by_url(self, url):
        result = self.db.items.find_one(
            {"url": url}
        )
        if not result:
            raise ItemNotFoundError()
        return Item(**result)

    def find_urls_by_healthcheck_name(self, name):
        items = []
        healthcheck = self.find_healthcheck_by_name(name)
        mgo_urls = self.db.items.find(
            {
                "group_id": healthcheck.group_id
            }, {"url": 1}
        )
        for url in mgo_urls:
            items.append(url['url'])
        return items

    def find_watchers_by_healthcheck_name(self, name):
        healthcheck = self.find_healthcheck_by_name(name)
        watchers = self.find_users_by_group(healthcheck.group_id)
        return [watcher.email for watcher in watchers]

    def remove_item(self, item):
        self.db.items.remove({"url": item.url})

    def add_user(self, user):
        self.db.users.insert(user.to_json())

    def remove_user(self, user):
        self.db.users.remove({"email": user.email})

    def add_healthcheck(self, healthcheck):
        self.db.healthchecks.insert(
            healthcheck.to_json()
        )

    def remove_healthcheck(self, healthcheck):
        self.db.healthchecks.remove(
            {"name": healthcheck.name}
        )

    def find_healthcheck_by_name(self, name):
        result = self.db.healthchecks.find_one(
            {"name": name}
        )
        if not result:
            raise HealthCheckNotFoundError()
        return HealthCheck(**result)

    def find_user_by_email(self, email):
        result = self.db.users.find_one(
            {"email": email}
        )
        if not result:
            raise UserNotFoundError()
        return User(result["id"], result["email"], *result["groups_id"])

    def find_users_by_group(self, group_id):
        items = self.db.users.find(
            {"groups_id": group_id},
        )
        return [User(r["id"], r["email"], *r["groups_id"]) for r in items]

    def add_user_to_group(self, user, group):
        self.db.users.update({"id": user.id}, {"$push": {"groups_id": group}})

    def remove_user_from_group(self, user, group):
        self.db.users.update({"id": user.id}, {"$pull": {"groups_id": group}})


class ItemNotFoundError(Exception):
    pass


class HealthCheckNotFoundError(Exception):
    pass


class UserNotFoundError(Exception):
    pass
